/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.pinot.core.query.reduce;

import org.apache.pinot.common.request.AggregationInfo;
import org.slf4j.LoggerFactory;


public class EqualComparison extends ComparisonFunction {
  private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(EqualComparison.class);
  private double _rightValue;

  public EqualComparison(String rightValue, AggregationInfo aggregationInfo) {
    super(aggregationInfo);
    try {
      this._rightValue = Double.parseDouble(rightValue);
    } catch (Exception e) {
      LOGGER.info("Exception in creating HAVING clause EQUAL predicate", e);
    }
  }

  @Override
  public boolean isComparisonValid(String aggResult) {
    try {
      double leftValue = Double.parseDouble(aggResult);
      if (leftValue == _rightValue) {
        return true;
      } else {
        return false;
      }
    } catch (Exception e) {
      LOGGER.info("Exception in applying HAVING clause EQUAL predicate", e);
      return false;
    }
  }

  public double getRightValue() {
    return _rightValue;
  }
}
